/* ***********************************************************************
**
**  Copyright (C) 2002  Jesper Hansen <jesperh@telia.com> and 
**			Romuald Bialy (MIS) <romek_b@o2.pl>.
**
**
**  Yampp-7/USB - main file
**
**  File yampp7_usb.c
**
*************************************************************************
**
**   This file is part of the yampp system.
**
**  This program is free software; you can redistribute it and/or
**  modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2
**  of the License, or (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software Foundation, 
**  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**
*************************************************************************
**
**  Revision History
**
**  when         what  who	why
**
**  2002-09-13   0.9   Jesper   
				-initial test release
**  2002-09-22   1.0   MIS	
				-change automatic power off time to 2 minutes (in stop mode)
				-added card present checking, if no card present or card removed - go to sleep mode
				-added LED output on PB0
				-changed wakeup from sleep - now we must press and hold "Play" key for two seconds
				-added some beeps and blinks
				-added beep during playlist change. Playlist <TOTAL> have different tone from other.
				-added little delay in keypress checking for avoid accidental key pressing.
				-added FastForward, Random, Loudness and Repeat functions.
				-all functions is handled by 4 keys, layout is depending on playing state. 
				-added trap for vs1001 lockup if MP3 has garbage
				-added manual power off -> longpress STOP key in Stop mode.
				-added auto power of from pause state after 30 minutes
				-added battery level monitoring.
				-added battery charge indicator. See README.TXT for details.
**  2002-09-25   1.10  MIS	
				-added true random function (max 1024 songs in playlist)
**  2002-10-04   1.11  MIS	
				-added card removing checking during pause and stop mode
				-code speed-up and cleaning
				-added configurable CF standby time (in Constants.h)
**  2002-10-07   1.12  MIS	
				-added keyboard locking functions with autolock. See README.TXT for details.
**  2002-10-09   1.13  MIS	
				-fixed accidentally bootloader entering during keyboard locking
				-fixed accidentally change song after keyboard unlocking
				-added beep for manual lock/unlock confirmation (may be off in Constants.h)
				-added autolock turn ON/OFF while normal operation (if enabled)
**  2002-10-14   1.20  MIS	
				-beta version, added suport for Nokia LCD (not public)
**  2002-10-18 to1.45  MIS	
				-most bugfix (not public)
**  2002-10-20   2.00  MIS	
				-Public beta version of y7 firmware with LCD support.
**  2002-10-21   2.10  MIS	
				-added total song time displaying.
				-minior bugfix.
**  2002-10-23   2.11  MIS	
				-some improvements and bugfixes in displaying functions
				-bugfix in next playlist change. Stack overvriting generate this problem.
				-little change in keylock icons
**  2002-10-29   2.12  MIS	
				-eeprom is written only before power-off or USB connect, for extend eeprom life.
				-added suport for card-specific graphical logo.
				-bugfix in battery charge indication during stop.
				-bugfix in artist name displaying if name is short.
				-bugfix in fast forward volume decrease.
**  2002-10-30   2.13  MIS	
				-added support for national characters displaying
**  2002-11-02   2.14  MIS	
				-added support for vs1001 equalizer
				-some memory usage fixes 
**  2002-11-05   2.15  MIS	-changed scroll routines
**  2002-11-23   2.16  MIS	-fixed card removing checking
				-fixed "MENU" text length
				-fixed battery level monitoring
				-fixed randomizer
				-alternate scroll is back
**  2002-12-18   2.17  MIS	
				-added remain song time displaying
**  2003-01-13   2.18  MIS    	
				-added compatibility with new versions of GCC
				-some menu functionality extensions
				-removed uart calls
				-removed not used beep calls
				-some code optimisations
**  2003-02-23   2.20  MIS   	
				-added MMC card support
				-added support for new version of VS1001 equalizer library with 12 presets
				-changes in ATA library
				-many code optimisations
				-fixed contrast setting

**  2003-03-18	2.21 jesper
				- modifications for new Rev. D board.
				- charge input changed to PE1 instead of PE2
				- expanded contrast range

**  2003-03-19	2.22   MIS	-small fixes in jesper modifications
				-bugfix in processing garbaged song
				-support for any old and a new Rev D board (see Constants.h)

**  2003-05-18	2.23   MIS	-added balance control
				-enabled playlist change during STOP state
				-added info about backlight mode
				-code size optimization
				-some bugfix in visualisation, charging control ect.

**  2003-07-14	2.25b1   MIS	-Sources changed for compilation with AVR-GCC 3.3 !!!
				-added new optional "PREVOUS" key functionality (work like in normal CD players).
				-fixed problems with some large CF cards
				-fixed problems with yamppLink conect/disconnect on MMC cards.
				-fixed Pause Mode

**  2003-08-08	2.25b2  nilrog	-fixed playlists handling by changing the "get_offset_entry" function.
			MIS	-adding way to fix the CF card reset after card inserting problem solving.

**  2004-01-18	2.28    MIS	-added support for native mode of AtMega162 with crystal about 6-14MHz.
				 This cause speed up file transfer to about 400kb/s (at 14.318Mhz)
				-removed not used Uart and Base libraries code and not used part in usb.c file.
				-added confirmation before card formatting.
				-fixed low battery checking
				-fixed startup without card
				-speedup MMC card write

**  2004-05-20	2.29    MIS	-added 6 backlight modes with dimmer in M162 mode
				-fixed VS1001 hardware reset if VS not respond
				-Fixed "Browse list" function.
				-Fixed MMC library

**  2004-10-08	2.30    Nilrog	-fixed repeat one song function
			MIS	-fixed MMC size and name detection
				-fixed standby high current consumption with some MMC cards

*********************************************************************** */

#include "yampp7_usb.h"
 
#define YAMPP_FIRMWARE_VERSION_MAJOR	2
#define YAMPP_FIRMWARE_VERSION_MINOR	30

#define DISK_LAYOUT_VERSION_MAJOR	1
#define DISK_LAYOUT_VERSION_MINOR	0


// The CPU is running with a 4.0 MHz (M161 CPU) or 4-15MHz (M162 CPU) clock

static char firmware_scan_tag[] __attribute__ ((progmem)) = "yfwTAG";
static char model_txt[]         __attribute__ ((progmem)) = "yampp-7";
static char date_txt[]          __attribute__ ((progmem)) = __DATE__" / "__TIME__;
static char version_txt[]       __attribute__ ((progmem)) = "2.30";

//------------------------------------------------------------------------------------------

extern u08 card_sbuf[512];

//------------------------------------------------------------------------------------------

void idle(void);
void next_song(u08 mode);

event_e menu_event;

u08 isPlaying;				// Playing status
u08 usb_play;				// usb playing status
u08 bLoudness;				// Loudness modificator
u08 bRandom;				// Random modificator
u08 Repeat;				// Repeat mode modificator
u08 keylock;				// Keylock status
u08 autolock;				// Autolock ON/OFF
u08 lobatt;				// Battery status
u08 timemode;				// Remain time modificator
u08 light_mode;				// LCD backlight mode

u08 volume;				// Volume level
s08 balance;				// Balance
u08 vol_decrease;			// Modificator for fast forward function
u16 ClusterCount;			// Counter for Fast Forward calculations
u16 addtime;				// for Fast Forward time calculation
u16 curPlaylist;			// Current playlist number
u16 curIndex;				// Current song number inside playlist
u32 filesize;				// size of file to play (divided by 32)
u32 fileplayed;				// amount of played data (divided by 32)
u16 bitrate;				// song bitrate for fast forward time calculation
u16 songtime;

u08 time_flag;				// general time counter
u08 scroll_flag;			// scrolling flag
u08 info_flag;
u08 light_time;				// LCD backlight time
u16 sleepcount;				// counter for automatic power off
u16 lobatt_time;			// counter for low battery checking
u16 keylock_time;			// automatic keylock timer

u16 vs1001_fw;				// Pointer into vs1001 additional firmware pointer
u08 *outptr;				// pointer for sending data to vs1001
u08 ARTISTNAME_BUF[48];			// buffer for artist name scrolling
u08 SONGNAME_BUF[55];			// buffer for title name scrolling
u08 *artistname = (u08*)ARTISTNAME_BUF;	// pointer to artistname buffer
u08 *titlename = (u08*)SONGNAME_BUF;	// pointer to songname buffer
u08 scroll_length;			// size of scroll data
u08 scroll_length_2;			// size of scroll data 2
u08 scroll_pos;				// current position in scroll string
u08 scroll_pos_2;			// current position in scroll string 2
#ifdef ALTERNATE_SCROLL
 u08 scroll_dir, scroll_dir_2;
 u08 scrollP, scrollP_2;
#endif

u08 fn0[] __attribute__ ((progmem)) = "Find Song";
u08 fn1[] __attribute__ ((progmem)) = "Next Playlist";
u08 fn2[] __attribute__ ((progmem)) = "Prev Playlist";
u08 fn3[] __attribute__ ((progmem)) = "Balance";
u08 fn4[] __attribute__ ((progmem)) = "Loudness";
u08 fn5[] __attribute__ ((progmem)) = "Random";
u08 fn6[] __attribute__ ((progmem)) = "Repeat Mode";
u08 fn7[] __attribute__ ((progmem)) = "Time Mode";
u08 fn8[] __attribute__ ((progmem)) = "LCD Contrast";
u08 fn9[] __attribute__ ((progmem)) = "Autolock Keys";

#if (BACKLIGHT_TIME != 0)
 u08 fn10[] __attribute__ ((progmem)) = "BacklightMode";
 u08 *function_name[] __attribute__ ((progmem)) = {
 (u08*)&fn0,(u08*)&fn1,(u08*)&fn2,(u08*)&fn3,(u08*)&fn4,(u08*)&fn5,(u08*)&fn6,(u08*)&fn7,(u08*)&fn8,(u08*)&fn9,(u08*)&fn10};
#else
 u08 *function_name[] __attribute__ ((progmem)) = {
 (u08*)&fn0,(u08*)&fn1,(u08*)&fn2,(u08*)&fn3,(u08*)&fn4,(u08*)&fn5,(u08*)&fn6,(u08*)&fn7,(u08*)&fn8,(u08*)&fn9};
#endif

//---------------------------------------------------------------------------

u08 CARD_SW_Reset2(void)					// ATA Reset
{
#ifdef MMC_CARD
	u08 retry = 3;						// retry counter

	while(--retry)
	{
		if(MMC_Reset())
			return 1;
	}
	return 0;

#else
	u16 timeout = 5000;					// timeout = 500ms

	WriteBYTE(ATAPI_DevCtrl, 0x06);				// SRST and nIEN bits
	delay10();						// 10uS delay
	WriteBYTE(ATAPI_DevCtrl, 0x02);				// nIEN bit
	delay10();						// 10 uS delay
	while ((ATA_ReadStat() & 0xC0) != SR_DRDY)		// Wait for DRDY and !BUSY
	{
		if (--timeout == 0)				// Timeout check
			return (0);				// Return with ERROR
		delay(100);					// 100 uS delay
	}
  	if (ReadBYTE(ATA_Sector) != 1)				// Checking for RESET signature
		return (0);					// Return with ERROR

	return 1;
#endif

}
//----------------------------------------------------------------------------

#define EVENT_QUEUE_SIZE	4
volatile event_e event_queue[EVENT_QUEUE_SIZE];
volatile u08 event_queue_head = 0;
volatile u08 event_queue_tail = 0;

void set_event(event_e e)
{
	event_queue[event_queue_head] = e;
	event_queue_head = (event_queue_head + 1) % EVENT_QUEUE_SIZE; 
}

event_e check_event(void)
{
	event_e tmp = EV_IDLE;
	
	//
	// check queue
	//
	if (event_queue_head != event_queue_tail)
	{
		tmp = event_queue[event_queue_tail];
		event_queue_tail = (event_queue_tail + 1) % EVENT_QUEUE_SIZE; 
	}
	return tmp;
}

event_e get_event(void)
{
	//
	// let the system have it's time
	//
	idle();
	return check_event();
}


//--------------------------------------------------------------------------
// States for	M162 mode	M161 mode
//
//    0  	Always OFF	Always OFF
//    1 	OFF/DIMM	OFF/ON
//    2		OFF/ON		Always ON
//    3		Always DIMM	not allowed
//    4		DIMM/ON		not allowed
//    5		Always ON	not allowed

#if (BACKLIGHT_TIME != 0)

void backlight_off(void)
{
#if defined (__AVR_ATmega162__)

	if(light_mode < 3)			// OFF state
		outp(0x00, OCR0);
	else if(light_mode < 5)
		outp(BL_DIMMLEVEL, OCR0);	// DIM state

#else

	if(light_mode < 2)
		cbi(PORTB,LED_PIN);		// OFF state

#endif
}

void backlight_on(void)
{
	light_time = 0;
#if defined (__AVR_ATmega162__)

	if(light_mode > 3 || light_mode == 2)
		outp(0xff, OCR0);		// ON state
	else if(light_mode != 0)
		outp(BL_DIMMLEVEL, OCR0);	// DIMM state

#else

	if(light_mode != 0)
		sbi(PORTB,LED_PIN);		// ON state

#endif
}

#endif // BACKLIGHT_TIME != 0

//--------------------------------------------------------------------------

//
// External INT2 (Play key)
//

SIGNAL(SIG_INTERRUPT2)				// Need this for wake up from sleep
{
	asm volatile ("nop");
}


//--------------------------------------------------------------------------

#if defined (__AVR_ATmega162__)
 #define F_CPU (M162CRYSTAL / M162DIVIDER)
#else
 #define F_CPU 4000000
#endif

#define TI1_H	(((u16)-(F_CPU / 2560)) >> 8)
#define TI1_L	(((u16)-(F_CPU / 2560)) & 0xff )

SIGNAL(SIG_OVERFLOW1)				// timer 1 overflow every 100 mS
{
	outp(TI1_H, TCNT1H);			// reload timer 
	outp(TI1_L, TCNT1L);
	time_flag++;				// general time counter
	scroll_flag++;
	nKeyTime++;				// counter for keyboard
	lobatt_time++;				// counter for low battery routine
	sleepcount++;				// sleep counter

#if (BACKLIGHT_TIME != 0)
	if (light_time != 255)
	{
		if (++light_time > BACKLIGHT_TIME)
		{
			light_time = 255;
			backlight_off();	// backlight OFF
		}
	}
#endif

	if (isPlaying && keylock==0 && autolock)
	{
		if (keylock_time <= AUTOLOCK_TIME+20)
			keylock_time++;
		else
		{
			keylock = 1;
#if (BACKLIGHT_TIME == 0)
			sbi(PORTB,LED_PIN);
#endif
		}
	}
}

//--------------------------------------------------------------------------

void LoudSet(void)
{
	if (vs1001_fw)
	{
		if(bLoudness)
		{
			vs1001_write(0x0A,vs1001_fw);
			vs1001_write(0x0D,0);
			vs1001_write(0x0D,bLoudness);
		}
		else
		{
			vs1001_write(0x0D,0);
			vs1001_write(0x0A,0);
		}
	}
	else
		vs1001_write(VS1001_MODE, 0x0080 * (bLoudness != 0));	// write bit SM_BASS to VS1001K

	vs1001_setvolume(volume,balance);
}

//--------------------------------------------------------------------------

static u08 buf[] __attribute__ ((progmem)) = {0x53, 0xEF, 0x6E, 0x45, 0x78, 0x69, 0x74};

void beep(u08 freq, u16 time)
{
	u08 i;
	vs1001_write(0x0D,0);
	vs1001_write(0x0A,0);
	if (isPlaying)
	{
		addtime += vs1001_read(VS1001_PLAYTIME);
		vs1001_reset(0);
	}
	vs1001_setvolume(volume+24,balance);		// -12dB

#if (BACKLIGHT_TIME == 0)
	sbi(PORTB,LED_PIN);				// LED on
#endif

	vs1001_nulls(8);
	for (i=0;i<3;i++)				// beep on
		vs1001_send_data(PRG_RDB(&buf[i]));
	vs1001_send_data(freq);
	vs1001_nulls(4);

	delayms(time);

	for (i=3;i<7;i++)				// sine OFF
		vs1001_send_data(PRG_RDB(&buf[i]));
	vs1001_nulls(4);

#if (BACKLIGHT_TIME == 0)
	cbi(PORTB,LED_PIN);				// LED Off
#endif

	LoudSet();					// restore Loudness settings and Volume
}
//--------------------------------------------------------------------------

void send_sinewave_beeps(void)
{
	u08 j;
	
	delayms(100);
	for (j=0;j<3;j++)
	{
		beep(SINE_1000,100);
		delayms(100);
	}
}

//--------------------------------------------------------------------------

void lcd_clrline(u08 line)		// clear single line of LCD
{
	register u08 i=14;
	lcd_gotoxy(0,line);
	for(i=0; i < 14; i++)
		lcd_putchar(' ');
	lcd_gotoxy(0,line);		// goto begin of line
}

//--------------------------------------------------------------------------
void status_display(void)
{
	u08 i = 0;
	lcd_gotoxy(0,5);
	
	if (bRandom)
	{
		i=127;
		lcd_invert();
	}
	lcd_wrdata(i);
	puts("\x1\x2");				// Random signs
	lcd_normal();
	lcd_wrdata(i);
	lcd_wrdata(0);

	lcd_putchar(Repeat + 0x10);		// Repeat sign L
	lcd_putchar(Repeat + 0x14);		// Repeat sign R	
	lcd_wrdata(0);

	if (vs1001_fw)
	{
		lcd_wrdata(0);
		lcd_putchar('L');
		lcd_putchar((bLoudness > 9) ? bLoudness + ('A'-10) : bLoudness + '0');
	}
	else
	{
		i=0;
		if (bLoudness)
		{
			i=127;
			lcd_invert();
		}
		puts("\x3\x4");			// MegaBass signs
		lcd_normal();
		lcd_wrdata(i);
	}

	lcd_wrdata(0);

	if ((keylock & 1) == 1)
	{
		puts("\x5\x6");			// locked sign

#if (BACKLIGHT_TIME != 0)
		backlight_off();		// Backlight off in locked state
#endif
	}
	else
	{
		if (autolock)
			puts("\xE\xF");		// autolock signs
		else
			puts("  ");
	}

	lcd_wrdata(0);

	if (lobatt > 1)
		puts("\x8\x9");			// low battery signs
	else
		puts("\xB\xC");			// good battery signs

	if(info_flag == 1)
	{
		i = balance;
		if(balance <= 0)
		{
			lcd_putchar(0x1A);
			i = 0-balance;
		}
		else
			lcd_putchar(' ');

		printf("%u",i/6);
		lcd_putchar( (balance >= 0) ? 0x19 : ' ');
	}
	else 
	{
		if (volume < MIN_VOLUME)
		{
			for (i=0; i<16; i++)
			{
				if(MIN_VOLUME - volume >= i * VOL_STEP * NUM_VOL_STEPS / 16)
					lcd_wrdata(0xff << (8 - (i+1)/2));
				else
					lcd_wrdata(0);
			}
		}
		else
		{
			vs1001_setvolume(254,0);
			lcd_wrdata(0);
			lcd_putchar(0x07);		// speaker
			lcd_wrdata(0);
			lcd_wrdata(0);
			lcd_putchar('x');		// off :)
		}
	}
}
//--------------------------------------------------------------------------

u08 lcd_card_logo(void)
{
	u16 i;
	if (CARD_SW_Reset2())
	{
		CARD_Read(YADL_SEC_SCRATCH);				// read scrath sector
		if (SectorBuffer[0] == 'y' && SectorBuffer[1] == 'W')	// check for Logo Tag
		{							// if present:
			lcd_gotoxy(0,0);				// begin at x=0, y=0
			for (i = 2; i < 4*84+2; i++)			// 4x84 bytes of logo bytes
				lcd_wrdata(SectorBuffer[i]);		// write to LCD
			return 0;					// return status: logo found
		}
	}
	return (lcd_logo(0));						// check for EEPROM logo
}

//--------------------------------------------------------------------------

void eeprom_update(void)
{
	union u16convert Data;
	// save current settings to EEPROM
	Data.value = curPlaylist;
	eeprom_wb(EEPROM_PLAYLIST, Data.bytes.low);
	eeprom_wb(EEPROM_PLAYLIST + 1, Data.bytes.high);
	Data.value = curIndex;					// save song number within current playlist
	eeprom_wb(EEPROM_SONGPOS, Data.bytes.low);
	eeprom_wb(EEPROM_SONGPOS + 1, Data.bytes.high);
	eeprom_wb(EEPROM_VOLUME,volume);
	eeprom_wb(EEPROM_BALANCE,balance);
	eeprom_wb(EEPROM_REPEAT,Repeat);
	eeprom_wb(EEPROM_RANDOM,bRandom);
	eeprom_wb(EEPROM_TIME,timemode);
	eeprom_wb(EEPROM_LOUDNESS, bLoudness);
	eeprom_wb(EEPROM_AUTOLOCK,autolock);
#if (BACKLIGHT_TIME != 0)
	eeprom_wb(EEPROM_BACKLIGHT,light_mode);		// save backlight mode
#endif
}

//--------------------------------------------------------------------------

void PowerOff(void)
{
	beep(SINE_500,150);
#if (BACKLIGHT_TIME != 0)
	backlight_on();
#endif
	lcd_clrscr();
	if(lcd_card_logo())
		lcd_gotoxy(2,2);
	else
		lcd_gotoxy(2,4);
	puts("Goodbye !!");
	u08 i =80;
	while(i--)
	{
		lcd_gotoxy(0,5);
		lcd_bar(i,82);
		delayms(15);
	}
	eeprom_update();
	sbi(ACSR,ACD);				// Disable Analog Comparator
	sbi(GIMSK,INT2);			// enable external interrupt 2 on falling edge
	wdt_disable();				// disable the watchdog
	sei();					// global interrupt enable
	cbi(PORTB, RESET_PIN);			// keep VS1001 reset
	lcd_clrscr();
	lcd_off();
	cbi(CHARGE_PORT,CHARGE_PIN);		// disable pullup on charge input
	cbi(CHARGE_PORT-1,CHARGE_PIN);		// and switch to input for no current draw in sleep mode
	outp(0, TCCR0);
	cbi(PORTB, LED_PIN);			// Backlight off

#ifdef MMC_CARD
	MMC_Off();
//	sbi(MMC_PORT,MMC_CS);			// MMC CS HIGH
//	sbi(MMC_PORT-1,MMC_CS);
#endif
	outp(0x00,PORTA);
	outp(0xff,DDRA);			// PORTA Output and LOW
	outp(0x80,PORTC);			// Deselect CF Card
	cbi(PORTB, PB2);			// FTDI reset
#ifndef FIXED_RESET
	outp(0x32,PORTD);			// WR and RD low to avoid powering FTDI chip via WR and RD signals
#else
	outp(0xB2,PORTD);			// RD high and and WR low to avoid powering FTDI chip via WR and RD signals,
#endif						// but need to modyfi the board. High RD is for solving problems
						// with accidentaly CF card "True IDE Mode" entering after card insert.

	outp(0xC0,DDRD);			// WR and RD as output
sldo:	outp(0x30,MCUCR);			// enable deep sleep mode and disable EXT RAM
	outp(0, TCCR1B);			// stop timer1
	asm volatile("sleep");			// Enter to Sleep mode
	asm volatile("nop");

// *********************************** 

	// wakeup going here
	outp(0x20,MCUCR);			// enable idle mode
	outp(4, TCCR1B);			// start timer1
	sleepcount = 0;
	while(sleepcount < 10)
	{
		asm volatile("sleep");		// Enter to idle mode
		asm volatile("nop");
		if bit_is_set(PINE,BUTTON_1)	// Play key pressed ?
			goto sldo;		// No, go to deep sleep again
	}
	outp(0xFF,PORTD);			// WR and RD high to avoid CF garbage
	wdt_enable(1);
	while (1);				// reset player by watchdog
}

/******************************************************************************
*				KEYBOARD CONTROL FUNCTION					*
******************************************************************************/

//
// button handling
//

//------------------------------------------------------------------------------------------
// Keyboard layout (while playing)
//
//		 			BUTTON_1	BUTTON_2	BUTTON_3	BUTTON_4	
u08 TableKbdShort[] = { EV_IDLE,	EV_PLAY,	EV_STOP, 	EV_NEXT,  	EV_PREV };
u08 TableKbdLong[]  = { EV_IDLE, 	EV_FFWD,	EV_MENU, 	EV_UP, 		EV_DOWN };

// Keyboard layout (On Stop)
//	 				BUTTON_1	BUTTON_2	BUTTON_3	BUTTON_4	
//			ShortPress:	PLAY		STOP	 	MBASS	  	RANDOM
//			LongPress: 	PWR.OFF 	MENU		AU.LOCK,	LCD Contrast


void check_keylock(void)
{
	if (autolock)
	{
		// if automatic lock timer enabled and timeout -> Led blink
		if (keylock_time > AUTOLOCK_TIME && keylock_time < AUTOLOCK_TIME+20)
			goto KBLINK;
	}

	if (bit_is_clear(PIND,BUTTON_3) && bit_is_clear(PIND,BUTTON_4))
	{						// if both top buttons down
		wdt_disable();				// disable watchdog for prevent from bootloader entering
		keylock |= 2;				// mark as pressed
#if (BACKLIGHT_TIME != 0)
		backlight_on();
#endif
		if (nKeyTime == 10)			// if pressed by 1 second
		{
			nKeyTime++;
			keylock ^= 1;			// flip keylock status (bit 0)
		}
		if (nKeyTime > 10)			// signalise by Led blink
		{
			nKeyTime = 11;

 KBLINK:

#if (BACKLIGHT_TIME == 0)
			if ((time_flag & 3) == 2)
			{
				time_flag++;
				if (bit_is_set(PORTB,LED_PIN))		// LED Flashing
					cbi(PORTB,LED_PIN);
				else
					sbi(PORTB,LED_PIN);
			}
#endif
			return;
		}
	}
	else
	{
		// both top buttons released and change occur
		if (keylock > 1 && bit_is_set(PIND,BUTTON_3) && bit_is_set(PIND,BUTTON_4))
		{
#if (BACKLIGHT_TIME == 0)
			sbi(PORTB,LED_PIN);		// Led ON
#endif
			nKeyTime = 0;			// clear key timer
			keylock_time = 0;		// clear automatic lock timer
			keylock &= 1;			// clear marker
			wdt_enable(6);			// enable watchdog
		}

		if (keylock)
			nKeyTime = 0;
	}
}

void get_key(void)
{
	static u08 keydown = 0;
	u08 keyflag;
	event_e event = EV_IDLE;

	if (keydown)					// if a key was pressed earlier
	{
		// see if it has been released
		
		if (bit_is_set(PINE,BUTTON_1) &&			// if key up
		    bit_is_set(PIND,BUTTON_2) &&			// if key up
		    bit_is_set(PIND,BUTTON_3) &&			// if key up
		    bit_is_set(PIND,BUTTON_4) )			// if key up
		{
			// no key is down, return last key seen
			if (nKeyTime < 8 && nKeyTime > 0)		// max 0.8 sec for shortpress
				event = TableKbdShort[keydown];	// short keypress event
			keydown = 0;
		}
		else
			if ((nKeyTime & 0x7F) >= 8)			// long keypress ? 
			{
#if (BACKLIGHT_TIME != 0)
				light_time = 0;
#endif
				event = TableKbdLong[keydown];		// long keypress event
				if (event == EV_UP || event == EV_DOWN || event == EV_FFWD)
					nKeyTime = 0x86;		// 0.2 sec repeat time for volume
				else
					nKeyTime = 0x80;		// as long repeat as possible
			}	
	}
	else
	{	
		keyflag = 0;
		if (bit_is_clear(PINE,BUTTON_1))			// if key 1 down
			keyflag = 1;					// store it
		if (bit_is_clear(PIND,BUTTON_2))			// if key 2 down
			keyflag = 2;					// store it
		if (bit_is_clear(PIND,BUTTON_3))			// if key 3 down
			keyflag = 3;					// store it
		if (bit_is_clear(PIND,BUTTON_4))			// if key 4 down
			keyflag = 4;					// store it

		if (keyflag && nKeyTime > 0)
		{
			keydown = keyflag;				// store it
			keylock_time = 0;

#if (BACKLIGHT_TIME != 0)
			backlight_on();
#endif
		}

		if (keyflag == 0)
			nKeyTime = 0;					// clear keypress timer
	}	

	if (event != EV_IDLE)
		set_event(event);
}

/******************************************************************
*			RANDOM STUFF				  *
******************************************************************/

// Random stuff, maximum 512 songs

static u32 seed = 1;
u08 RANDOM_TAB[64];				// "True Random" table, maximum 512 songs

// Init randomize table
void InitRnd(void)
{
u08 Temp;
	if (bRandom)
	{
		for (Temp = 0; Temp < 64; Temp++)
			RANDOM_TAB[Temp] = 0;		// one bit in table represent one song
	}
}


u16 randcalc(u16 max)				// returned random value from 0 to max - 1
{
//  return ((seed = seed * 1103515245L + 12345) % max);	// get new song number

	asm volatile ("push r24");
	asm volatile ("push r25");
	asm volatile ("lds r22,seed");
	asm volatile ("lds r23,(seed)+1");
	asm volatile ("lds r24,(seed)+2");
	asm volatile ("lds r25,(seed)+3");
	asm volatile ("ldi r18,lo8(1103515245)");
	asm volatile ("ldi r19,hi8(1103515245)");
	asm volatile ("ldi r20,hlo8(1103515245)");
	asm volatile ("ldi r21,hhi8(1103515245)");
	asm volatile ("call __mulsi3");
	asm volatile ("subi r22,lo8(-(12345))");
	asm volatile ("sbci r23,hi8(-(12345))");
	asm volatile ("sbci r24,hlo8(-(12345))");
	asm volatile ("sbci r25,hhi8(-(12345))");
	asm volatile ("sts seed,r22");
	asm volatile ("sts (seed)+1,r23");
	asm volatile ("sts (seed)+2,r24");
	asm volatile ("sts (seed)+3,r25");
	asm volatile ("pop r19");
	asm volatile ("pop r18");
	asm volatile ("clr r20");
	asm volatile ("clr r21");
	asm volatile ("call __udivmodsi4");
	asm volatile ("mov r25,r23");
	asm volatile ("mov r24,r22");
	return max;
}


u08 tabval(u16 num, u08 fn)				// check random tabele for song status
{								// and mark as played if necessary
	u08 index = (u08)(num >> 3);
	u08 offset = 1 << (num % 8);
	if(fn)
		RANDOM_TAB[index] |= offset;		// if fn=1 mark as played
	return (RANDOM_TAB[index] & offset);	// return status
}

// new random procedure
u16 do_rand(u16 max)					// returned random value from 0 to max - 1
{
u16 num;
	max &= 0x01ff;					// limit to 512 songs for lower RAM usage
	for(num = 0; num < max; num++)		// check for all songs in playlist
		if(tabval(num,0) == 0)			// not be played yet
			break;
	if (num == max)
		num = -1;					// no more songs in current playlist
	else
	{
		do
		{
			wdt_reset();
			num = randcalc(max);
		} while (tabval(num,0) != 0);		// check this song not be played piervously
		tabval(num,1);				// mark as just played
	}
	return num;
}


/************************************************************************
*				FORMAT NEW DISK				*
*************************************************************************/

u08 format_new_disk(void)
{
	YADL_ROOT_SECTOR *pBoot;					// Boot Sector struct
	u32 sectors;

#ifndef MMC_CARD

	IDENTIFY_DATA *pID;	
	u08 i,c;
	u08 tmpSectorBuffer[sizeof(pID->ModelNumber)];

	// first get some drive data
	CARD_Identify();
	
	// flip and save the ModelNumber data
	pID = (IDENTIFY_DATA *) SectorBuffer;
	strncpy(tmpSectorBuffer,(u08*)pID->ModelNumber,sizeof(pID->ModelNumber));
	for (i=0;i<sizeof(pID->ModelNumber);i+=2)
	{
		c = tmpSectorBuffer[i];
		tmpSectorBuffer[i] = tmpSectorBuffer[i+1];
		tmpSectorBuffer[i+1] = c;
	}
	tmpSectorBuffer[sizeof(pID->ModelNumber)] = 0;			// terminate entry
	
	// get max sectors (disk size)
	sectors = pID->UserAddressableSectors;

#else

	u08 tmpSectorBuffer[8];
	MMC_Name();
	strncpy(tmpSectorBuffer, SectorBuffer, 7);
	tmpSectorBuffer[7] = 0;					// terminate entry
	sectors = MMC_Capacity();

#endif

	// clear buffer
	pBoot = (YADL_ROOT_SECTOR *) SectorBuffer;		// use temp buffer for room
	strcpy_P(pBoot->model,firmware_scan_tag);		// for compiler warning supression...
								// I don't know why, but it decrease code size too
	memset(pBoot,0,512);				

// section 1
	strncpy(pBoot->id,"YADL",4);
	pBoot->disk_size = sectors;
	pBoot->unmount_state = YADL_STATE_NEW;
	pBoot->media_type = YADL_MEDIA_FLASH;

	pBoot->disk_version_major = DISK_LAYOUT_VERSION_MAJOR;
	pBoot->disk_version_minor = DISK_LAYOUT_VERSION_MINOR;

	strcpy_P(pBoot->model,model_txt);	// yampp model name

	// copy the ModelNumber result from Identify Drive
	strcpy(pBoot->disk_info,tmpSectorBuffer);	

// section 2
	
	strcpy_P(pBoot->build_timestamp,date_txt);
	pBoot->build_timestamp[YADL_LEN_BUILD_TIMESTAMP-1] = 0;	// make sure it's terminated

	pBoot->firmware_version_major = YAMPP_FIRMWARE_VERSION_MAJOR;
	pBoot->firmware_version_minor = YAMPP_FIRMWARE_VERSION_MINOR;
	
// section 3	
// section 4
// section 5

	// write the sector
	// and return result
	
	return CARD_Write(YADL_SEC_ROOT);
}



//////////////////////////////////

u32 songbase_start;
u16 songbase_qty;			// number of songs
u32 fat_start;				// first fat sector
u32 sector_cluster_offset;		// add to cluster -> sector conversions
u32 playlist_start;			// first playlist sector
u16 playlist_qty;			// number of playlists

/////////////////////////////////

u32 MyCluster;				// current cluster number
u32 MySector = 2;			// current sector number
u16 SectorsLd;				// sectors in cluster counter

//
// return the next FAT in a chain
//

u32 next_fat(void)
{
	u32 fat_sector = fat_start + MyCluster / (SECTORSIZE / sizeof(CLUSTER));
	u16 fat_offset = MyCluster % (SECTORSIZE / sizeof(CLUSTER));
	while (CARD_Read(fat_sector))
	    	CARD_SW_Reset2();
	ClusterCount++;						// counter for F.Forward file sizing
	return ((CLUSTER*)SectorBuffer)[fat_offset];
}


void init_load_sectors(u32 cluster)
{
	MyCluster = cluster;
	MySector = sector_cluster_offset + YADL_SECTORS_PER_CLUSTER * MyCluster;
	SectorsLd = 0;
}


//
// load a bufferfull of data
//
void load_sector(void)
{
	if (SectorsLd == YADL_SECTORS_PER_CLUSTER)
		init_load_sectors(next_fat());
	while(CARD_Read(MySector))
	    	CARD_SW_Reset2();
	MySector++;
	SectorsLd++;
}

/****************************************************************************************/

void init_stuff(void)
{
	u08 i,update = 0;
	YADL_ROOT_SECTOR *pRoot;				// Boot Sector struct
	pRoot = (YADL_ROOT_SECTOR *) SectorBuffer;		// Allocate room from the temp buffer

	while (CARD_Read(YADL_SEC_ROOT))			// read root sector
		CARD_SW_Reset2();

	if(pRoot->id[0] != 'Y' || pRoot->id[1] != 'A' || 
	   pRoot->id[2] != 'D' || pRoot->id[3] != 'L')		// YADL signature not found
	{
		lcd_clrscr();
		puts("Unknown Card,\nPress STOP for\nformat to YADL\n");
		static	u08 tones[]= {SINE_300,SINE_500,SINE_800,SINE_1000};
		for (i=0; i<4; i++)
		{		
			beep(tones[i],200);
			delayms(10);
		}
		do{
			get_key();
			i = check_event();
			wdt_reset();
		}while(i != EV_STOP);
		if(format_new_disk() == 0)			// format Card with YADL f.s.
		{
			puts("Formatting OK.");
			beep(SINE_1500,100);
		}
		else
		{
			puts("Format ERROR !");
			beep(SINE_500,200);
		}
		wdt_enable(6);					// time for text displaying
		while(1);					// reset player by watchdog
	}
	if(pRoot->firmware_version_major != YAMPP_FIRMWARE_VERSION_MAJOR)
	{
		pRoot->firmware_version_major = YAMPP_FIRMWARE_VERSION_MAJOR;
		update = 1;
	}
	if(pRoot->firmware_version_minor != YAMPP_FIRMWARE_VERSION_MINOR)
	{
		pRoot->firmware_version_minor = YAMPP_FIRMWARE_VERSION_MINOR;
		update = 1;
	}
	if(strcmp_P(pRoot->build_timestamp, date_txt))
	{
		strcpy_P(pRoot->build_timestamp, date_txt);
		update = 1;
	}
	if (update)
	{
		while(CARD_Write(YADL_SEC_ROOT))
		    	CARD_SW_Reset2();
		lcd_clrscr();
		puts("\nFirmware vers.n\ninfo updated!\n");
		delayms(100);
		beep(SINE_1500,80);
		beep(SINE_2000,80);
		delayms(1000);
	}

	sector_cluster_offset = pRoot->sector_cluster_offset;
	songbase_start 	= pRoot->songbase_start;			// SECTOR !!
	songbase_qty	= pRoot->songbase_qty;
	playlist_start 	= sector_cluster_offset + YADL_SECTORS_PER_CLUSTER * pRoot->playlist_start;	// SECTOR !!!
	playlist_qty 	= pRoot->playlist_qty;		
	fat_start 	= pRoot->fat_start;

	//
	// VS1001 User Software Loader
	//

	u32 app_start = pRoot->wasteland_start;
	vs1001_fw = 0;
	if (app_start != 0)
	{
		u16 *pData;
		pData = (u16*) SectorBuffer;
		CARD_Read(app_start);						// read first free sector
		if(*pData++ != 0x4679 || *pData++ != 0x776D)			// Firmware signature ('yFmw') not found
			return;							// exit
		u16 DataLen = *pData++;						// length of firmware in words (firmware only)
		vs1001_fw = *pData++;						// address to enter into VS1001 register A1ADDR
										// for application running.
		vs1001_write(7,vs1001_fw + 0x4000);				// write to VS1001 register WRAMADDR
		while(DataLen--)						// data length in words
		{
			if (pData == (u16*)(SectorBuffer+512))
			{
				app_start++;
				CARD_Read(app_start);				// read next sector with application datas
				pData = (u16*) SectorBuffer;			// new start address
			}
			vs1001_write(6,*pData++);				// write firmware word to vs1001
		}
	}
}

/****************************************************************************************/

YADL_PLAY_LIST_ENTRY *get_pl_entry(u16 index)
{
	u32 nCurSector;
	
	//
	// find (and load) correct playlist sector
	//
	nCurSector =  playlist_start + (index >> 3);	// 64 bytes per one playlist entry
	while(CARD_Read(nCurSector))
		CARD_SW_Reset2();
	return &((YADL_PLAY_LIST_ENTRY *)SectorBuffer)[index % 8];
}

YADL_SONG_BASE_ENTRY *get_song_entry(u16 index)
{
	u32 nCurSector;
	
	nCurSector = songbase_start + (index >> 2);
	while(CARD_Read(nCurSector))
		CARD_SW_Reset2();
	return &((YADL_SONG_BASE_ENTRY *)SectorBuffer)[index % 4];
}


// Nilrog: Rewrote this function since it couldn't properly handle
//         the fact that a playlist can begin in one sector and the
//         index pointed to a position which was located in the next
//         sector.

u16 get_offset_entry(u16 index)
{
 	YADL_PLAY_LIST_ENTRY *pListEntry = get_pl_entry(curPlaylist);
 	FOFFSET offset = pListEntry->offset;
	
	/* calculate absolute position from start of playlistfile */
	u16 pos = offset + index * sizeof(u16);

	/* calculate if the current playlist position wraps into another sector */
	u08 wraps = pos / SECTORSIZE;

	/* calculate which sector to read */
	u32 nCurSector = playlist_start + wraps;

	/* if playlist wraps we need to re-calculate index and offset */
	if (wraps)
	{
		index = (pos - wraps * SECTORSIZE) / sizeof(u16);
		offset = 0;
	}
	/* if no wrap ocurred 'index' and 'offset' are clear to use as is */

 	while(CARD_Read(nCurSector))
 		CARD_SW_Reset2();
 
 	u16 *pList = (u16*) ( (u08*)SectorBuffer + offset);
 	return pList[index];
 }


//********************************************************************************

void Playlist_name(void)
{
	u08 *src;
	udiv_t divt;
	YADL_PLAY_LIST_ENTRY *pListEntry = get_pl_entry(curPlaylist);
	u16 tmp = (pListEntry->entry_qty);
	lcd_clrscr();
	lcd_invert();
	u08 j = strlen(pListEntry->name);
	if (j<14)
	{
		j = (14-j)*3;
		while(j--)
			lcd_wrdata(127);
	}
	lcd_puts(pListEntry->name);			// playlist name
	j=8;
	while(j--)
		lcd_putchar(' ');
	lcd_normal();

	YADL_SONG_BASE_ENTRY *pEntry = get_song_entry(get_offset_entry(curIndex));
	strcpy(titlename, pEntry->title);
	src = titlename + strlen(titlename);
	*src++ = ' ';
	*src++ = '(';
	songtime = pEntry->play_time;
	divt = udiv(songtime, 600);
	u08 s = (u08)divt.quot % 10;
	if (s)
		*src++ = '0' + s;				// song time up to 99 min
	divt = udiv(divt.rem, 60);
	*src++ = '0' + (u08)divt.quot;
	*src++ = ':';
	divt = udiv(divt.rem, 10);
	*src++ = '0' + (u08)divt.quot;
	*src++ = '0' + (u08)divt.rem;			
	*src++ = ')';

#ifndef ALTERNATE_SCROLL
	if (src-titlename > 14)
		*((u32*)src)++ = 0x203E3E20L;		// adding " >> " sign
#else
	scroll_dir = scroll_dir_2 = 0;
	scrollP = scrollP_2 = 0;
#endif
	*src = 0;
	scroll_length_2 = src-titlename;
	scroll_pos = scroll_pos_2 = 0;

	printf("\n%s    \n%s\n#%u/%u", pEntry->artist, titlename, curIndex+1, tmp);
	status_display();
	CARD_Read(MySector-1);
	time_flag = 5;
	scroll_flag = 2;
}

// ******************************************************************************

// this is in assembler procedure because c not propper handle 
// pointers array located in program memory (in this case)
u08* PRG_RDW(u08* addr)
{
	asm volatile ("movw r30,r24");
	asm volatile ("LPM  r24,Z+");
	asm volatile ("LPM  r25,Z");
	return addr;
}

#define SONGS_D	0
#define MENU_D	1

u16 browse_list(u16 index, u16 min, u16 max, u08 func)
{
	u08 i,j = 0;
	event_e event;
	while (1)
	{
		for (i=0; i < 2 + 2*func; i++)
		{
			if (func)
			{
				lcd_clrline(1 + i);
				if (index+i < max)
				{
					lcd_putchar((i==j) ? 0x18 : ' ');
	 				lcd_progputs(PRG_RDW((u08*)&function_name[index+i]));	// menu print on LCD
				}
			}
			else
			{
				lcd_clrline(2 + 2*i);
				lcd_clrline(1 + 2*i);
				if (index+i < max)
				{
					YADL_SONG_BASE_ENTRY *pEntry = get_song_entry(get_offset_entry(index+i));
					lcd_putchar((i==j) ? 0x18 : ' ');
					lcd_puts(pEntry->artist);
					lcd_putchar('\n');
					lcd_putchar((i==j) ? 0x18 : ' ');
					lcd_puts(pEntry->title);		// print title
				}
				CARD_Read(MySector-1);
				idle();
			}
		}

		do 
		{
			time_flag = 0;
			sleepcount = 0;						// power off blocking
			event = get_event();
			switch (event)
			{
				case EV_NEXT:
				case EV_UP:
					if ( index+j > min )
					{
						if(j)
							 j--;
						else
							index--;
					}
					break;
				case EV_PREV:
				case EV_DOWN:
					if ( index+j+1 < max )
					{
						if(j < 2*func + 1)
							j++;
						else
							index++;
					}
					break;
				case EV_PLAY:
					info_flag = 3;
					return index+j;

				case EV_STOP:
					info_flag = 3;
					return -1;
			}
		}
		while (event == EV_IDLE);
	}
}


// ***************************************************************************************

u16 browse_songs(void)
{
	lcd_gotoxy(0,0);
	lcd_invert();
	puts(" SELECT SONG: ");
	lcd_normal();
	YADL_PLAY_LIST_ENTRY *pListEntry = get_pl_entry(curPlaylist);
	return (browse_list(curIndex, 0, pListEntry->entry_qty, SONGS_D));
}


/***********************************************************************************/

u16 browse_menu(void)
{
	lcd_gotoxy(0,0);
	lcd_invert();
	u08 i = 3;
	while(i--)
		lcd_wrdata(127);
	puts(" - M E N U -  ");
	lcd_normal();
	return (browse_list(menu_event ? menu_event-1 : 0, 0,10 + (BACKLIGHT_TIME != 0),MENU_D));
}

/***********************************************************************************/
#ifdef PCB_REV_D

u08 IsCharging(void)
{
	u08 b = 0;
	u08 c = inp(MCUCR);
	
	outp(0, MCUCR);				// disable external memory interface 

	cbi(CHARGE_PORT-1,CHARGE_PIN);		// set pin as input
	sbi(CHARGE_PORT,CHARGE_PIN);		// activate pullup

	asm volatile("nop");
	asm volatile("nop");
	if (bit_is_clear(CHARGE_PORT-2,CHARGE_PIN))	// if charging 
		b = 1;
	outp(c, MCUCR);				// restore control register setting
			
	return b;
}

#else // PCB older than D

u08 IsCharging(void)
{
	return (bit_is_clear(CHARGE_PORT-2,CHARGE_PIN));
}


#endif // PCB_REV_D


void BatteryMeasure(void)
{
	if (!IsCharging())					// no charging in progress
	{
		cbi(PORTB, PB3);				// PB3 as analog comparator input
		cbi(DDRB, PB3);					// for battery level measure
		delay(80);
		if (bit_is_set(ACSR,ACO))			// low battery
		{
			if (!lobatt)
			{
				lobatt = 1;
				lobatt_time = 0;
			}
			else
				lobatt = 2;

		}
		sbi(PORTB, PB3);
		sbi(DDRB, PB3);
	}
}


//********************************************************************************
//
// Run this as often as possible
// preferably every time around the main loop
//
void idle(void)
{
static	u08 ledflash = 1;
	u08 ContWrites = 250;					// counter for prevent infinite vs1001 writing
	u08 offtime = 20;
	u08 ontime = (lobatt > 1) ? 8 : 1;

	wdt_reset();						// reset the watchdog
	usb_handler();
	if(usb_link == 0)					// usb link disables other control
	{
		if (isPlaying) // && !sleepenabled)		// only during playing
			check_keylock();
		else
			keylock = 0;

		if (!keylock)					// if keylock active
			get_key();				// check keyboard
	}
	else
		return;

	if (isPlaying)
	{
		sleepcount = 0;					// power off blocking
		// pump some data to the decoder
		while (bit_is_set(PIND, DREQ_PIN))		// while DREQ is hi 
		{
			vs1001_send32(outptr);			// send data on SPI bus
			outptr += 32;				// advance data pointer
			fileplayed++;

			if (--ContWrites == 0)			// if vs1001 needs to more data
			{
				wdt_reset();			// reset the watchdog
				cbi(PORTB, RESET_PIN);		// VS1001 hardware reset
				delayms(1);
				sbi(PORTB, RESET_PIN);		// VS1001 reset
				delayms(10);
				vs1001_reset(0);
				LoudSet();
				ContWrites = 250;
				break;				// let's take some time for system
			}

			// check for buffer wrap
			if ( outptr == (SectorBuffer+512) )	// buffer empty
			{
				outptr = SectorBuffer;		// data pointer at begin of sector buffer

				if (!usb_play)
					load_sector();		// will load into SectorBuffer on YAMPP7
				else
				{
					usb_response(0);
					break;
				}
			}
			
			// check if we're done

			if (fileplayed >= filesize)
			{
				if (!usb_play)
					isPlaying = 0;		// the end is near
				next_song(0);
				return;
			}
		}

		if (vol_decrease && (time_flag == 4))		// If fast forward volume downed
		{
			vs1001_setvolume(volume,balance);	// Restore volume
			vol_decrease = 0;
		}


//------------------------ LCD VISUALISATION ----------------------------

		if (scroll_flag >= SCROLL_SPEED && info_flag < 2)
		{
			u08 nTemp;
#ifndef ALTERNATE_SCROLL
			static u08 scrollP;
			#define scrollP_2 scrollP
#endif
			scroll_flag = 0;
			//
			// scrolling artist and name

			if (scroll_length > 14)
			{
				lcd_gotoxy(0, 1);
				lcd_data_o(artistname[scroll_pos % scroll_length],scrollP);
				for (nTemp = 1; nTemp <  15; nTemp++)
#ifdef ALTERNATE_SCROLL
					lcd_putchar(artistname[scroll_pos + nTemp]);

				if (scroll_pos + 14 == scroll_length)
					scroll_dir -= 2;
				if (scroll_pos == 0 && scrollP == 0)
					scroll_dir++;

				scrollP += ((signed char)scroll_dir / (10/SCROLL_SPEED)) * SCROLL_STEP;
				if (scrollP == 6)
				{
					scrollP = 0;
					scroll_pos++;
				}
				if (scrollP > 127)
				{
					scrollP = 6 - SCROLL_STEP;
					scroll_pos--;
				}

#else //ALTERNATE_SCROLL
					lcd_putchar(artistname[(scroll_pos + nTemp) % scroll_length]);
#endif
			}

			lcd_gotoxy(0, 2);
			if (scroll_length_2 > 14)
			{
				lcd_data_o(titlename[scroll_pos_2 % scroll_length_2],scrollP_2);
				for (nTemp = 1; nTemp < 15; nTemp++)
#ifdef ALTERNATE_SCROLL
					lcd_putchar(titlename[scroll_pos_2 + nTemp]);

				if (scroll_pos_2 + 14 == scroll_length_2)
					scroll_dir_2 -= 2;
				if (scroll_pos_2 == 0 && scrollP_2 == 0)
					scroll_dir_2++;

				scrollP_2 += ((signed char)scroll_dir_2 / (10/SCROLL_SPEED)) * SCROLL_STEP;
				if (scrollP_2 == 6)
				{
					scrollP_2 = 0;
					scroll_pos_2++;
				}
				if (scrollP_2 > 127)
				{
					scrollP_2 = 6 - SCROLL_STEP;
					scroll_pos_2--;
				}
#else //ALTERNATE_SCROLL
					lcd_putchar(titlename[(scroll_pos_2 + nTemp) % scroll_length_2]);
#endif
				lcd_gotoxy(0,3);
				lcd_data('#');
			}
			else
				lcd_puts(titlename);

#ifndef ALTERNATE_SCROLL
			scrollP += SCROLL_STEP;
			if (scrollP == 6)
			{
				scrollP = 0;
				scroll_pos++;
				scroll_pos_2++;
			}
#endif
		}

		if (time_flag >= 5 && info_flag < 2)
		{
			if(bLoudness)
				vs1001_write(0x0D,bLoudness);			// Rewrite loudness settings
										// if VS1001 do selfreset
			//
			// display bar function

			register u08 nSteps = (u08)((fileplayed * 82) / filesize);
			lcd_gotoxy(0,4);
			lcd_bar(nSteps,82);
			status_display();

			//
			// printing song time 

			lcd_gotoxy(8,3);					// time display position
			u16 wPlayTime = vs1001_read(VS1001_PLAYTIME) + addtime;

			if (timemode == 1)
			{
				wPlayTime = songtime - wPlayTime;		// calculate time remain
				if ((s16)wPlayTime < 0)
					wPlayTime = 0;
			}
			printf("%3u:%02u", wPlayTime / 60, wPlayTime % 60); 	// print time on LCD
			if (timemode == 1)
			{
				lcd_gotoxy(8 + (wPlayTime < 600),3);
				lcd_putchar('-');
			}

			if (IsCharging())				// charging in progress ?
			{
				lobatt = 0;
				lobatt_time = 0;
				ledflash = !ledflash;
				lcd_gotoxy(9,5);
				lcd_wrdata(0);
				if (ledflash)
					puts("  ");
				else
					puts("\xB\xC");
			}
			if (!info_flag)
				time_flag = 0;
		}

		if((time_flag > 30) && (menu_event != EV_IDLE))		// display overvrite timer ends
		{
			info_flag = 0;					// enable status updating
			menu_event = EV_IDLE;
			Playlist_name();				// show Playlist name
		}

//------------------------ LCD VISUALISATION END -------------------------

	}//if (isPlaying)
	else
	{
		if (IsCharging())				// charging in progress ?
		{
			sleepcount = 0;				// power off blocking
			ontime = 20;				// LED 2 sec on / 0.2 sec off
			offtime = 2;				// until end charge.
			lobatt = 0;				// clear low battery status
			lobatt_time = 0;			// clear low battery timer

			if (time_flag % 10 == 0)
			{
				if(!info_flag)			// not in menu
				{
					lcd_gotoxy(0,4);
					puts("Battery charge");
				}
				lcd_gotoxy(9,5);
				lcd_wrdata(0);
				puts("  ");
			}

			if (time_flag % 10 == 5)
				status_display();
		}

#if (BACKLIGHT_TIME == 0)
		if (ledflash-- == 0)
		{
			if (bit_is_set(PORTB,LED_PIN))	// LED Flashing
			{
				cbi(PORTB,LED_PIN);		// off
				ledflash = offtime;
			}
			else
			{
				sbi(PORTB,LED_PIN);		// on
				ledflash = ontime;
			}
		}
#endif
		if (!menu_event)
		{
			outp(0xA0,MCUCR);			// choose idle mode
			asm volatile ("sleep");			// go to sleep
			asm volatile ("nop");
			wdt_reset();				// reset the watchdog
		}

		if (sleepcount > DEEP_SLEEP_TIMEOUT)
			PowerOff();				// Go to power off procedure and off player

		if (! CARD_SW_Reset2())				// if card removed
			PowerOff();				// power off

		if (time_flag >= 30)
		{
			if (menu_event || (lobatt_time < 50 && offtime == 20))	
				set_event(EV_STOP);		// if auto menu exit or end of battery charge
			else
				status_display();

			menu_event = EV_IDLE;
			time_flag = 0;
		}
	} 

	BatteryMeasure();
	
	u16 off_time = lobatt_time;
	if (lobatt == 2)
	{

#if (BACKLIGHT_TIME == 0)
		if (isPlaying && (off_time % 20) == 0)		// while playing
			cbi(PORTB,LED_PIN);			// and battery level is low

		if (isPlaying && (off_time % 20) == 10)		// led blinks
			sbi(PORTB,LED_PIN);
#endif
		if ((off_time % 600) == 10)			// and beeps in one minute interval
			beep(SINE_2000,80);
		
		if (off_time > 6050)				// and power off after 10 minutes
		{
			send_sinewave_beeps();
			PowerOff();
		}
	}
}

void play_song(void)
{
	isPlaying = 1;
	if (info_flag == 0)
		Playlist_name();

	YADL_SONG_BASE_ENTRY *pEntry = get_song_entry(get_offset_entry(curIndex));

	if(pEntry->state != YADL_STATE_READY)			// check for valid song entry
	{
		puts("\nSongBase Error !");
		beep(SINE_500,500);
		set_event(EV_STOP);
		return;
	}

	// init randomizer from timer
	seed += (inp(TCNT1H)<<8) + (inp(TCNT1L)) + 1;

	addtime = 0;
	bitrate = ((pEntry->bitrate) & 0x7fff);			// get bitrate and mask VBR signature (MSB)
	strcpy(artistname, pEntry->artist);
	scroll_length = strlen(artistname);
#ifndef ALTERNATE_SCROLL
	if (scroll_length > 14)
	{
		strcat(artistname, " >> ");			// add ' >> ' sign
		scroll_length += 4;		
	}
#endif

	//
	// calc number of VS1001 blocks
	//

//	u08 garbage = ((pEntry->bitrate & (~OPTION_BR_ID1_TAG)) != 0) ? 128 : 0;
//	filesize = (pEntry->length - garbage) / 32;
								// calculation number of blocks, 14 bit of
								// bitrate indicate that file have ID3 tag
								// and this may be excluded (last 128 bytes)
	filesize = pEntry->length / 32;

	fileplayed = 0;
	ClusterCount = 0;					// Reset Cluster counter

	//
	// preload buffers
	//
	init_load_sectors(pEntry->start);

//	u08 i;
//	for (i=0; i < (pEntry->start_offset / SECTORSIZE); i++)
//		load_sector();						// skip first pEntry->start_offset of full buffers

	outptr = SectorBuffer; // + (pEntry->start_offset % SECTORSIZE);	// and rest of pEntry->start_offset

	//
	// start playing the song
	//

#if (BACKLIGHT_TIME == 0)
	sbi(PORTB, LED_PIN);					// LED ON
#endif
	vs1001_reset(0);
	LoudSet();
}

void next_playlist(void)
{
	YADL_PLAY_LIST_ENTRY *pListEntry = get_pl_entry(curPlaylist);

	if(pListEntry->next_playlist != (u16)-1)		// check for next playlist number
		curPlaylist = pListEntry->next_playlist;
	else
		curPlaylist++;

	if((curPlaylist == (u16)-1) || (curPlaylist >= playlist_qty))
		curPlaylist = 0;

	pListEntry = get_pl_entry(curPlaylist);
	InitRnd();							// init randomizer table

	curIndex = (bRandom) ? do_rand(pListEntry->entry_qty) : 0;
}

void prev_playlist(void)
{
	if(--curPlaylist == (u16)-1)					// playlist down
		curPlaylist = playlist_qty - 1;				// go to last playlist
	YADL_PLAY_LIST_ENTRY *pListEntry = get_pl_entry(curPlaylist);	// update playlist data
	InitRnd();

	curIndex = (bRandom) ? do_rand(pListEntry->entry_qty) : pListEntry->entry_qty-1;
}

void next_song(u08 mode)
{
	YADL_PLAY_LIST_ENTRY *pListEntry = get_pl_entry(curPlaylist);

	if (Repeat == 3 && mode == 0)				// repeat one song
		goto GO_PLAY;


	if (mode == 2)
	{
#ifdef NEW_PREW
		if(ClusterCount)
			goto GO_PLAY;
		else
#endif
			curIndex = (bRandom) ? do_rand(pListEntry->entry_qty) : curIndex-1;
	}
	else
		curIndex = (bRandom) ? do_rand(pListEntry->entry_qty) : curIndex+1;

	if (curIndex < (pListEntry->entry_qty))			// if not End of playlist
		goto GO_PLAY;
	else
	{
		InitRnd();					// init randomizer table
		if(Repeat == 2)					// repeat playlist
		{
			if (bRandom)
				curIndex = do_rand(pListEntry->entry_qty);
			else
				curIndex = 0;
			goto GO_PLAY;
		}

		if(Repeat == 0)
		{
			set_event(EV_STOP);			// normal play, stop after end of playlist
			return;
		}

		if(Repeat == 1 || Repeat == 3)			// normal play and repeat one song, next playlist after end of playlist
		{						// fixed by Roger Nilson
			if (mode == 2)
				prev_playlist();
			else
				next_playlist();
GO_PLAY:		play_song();
		}
	}
}

//----------------------------------------------------------------------------

void set_contrast(void)
{
	event_e event = EV_IDLE;
	u08 contrast = eeprom_rb(EEPROM_LCD_CONTRAST);
	lcd_clrscr();
	puts("Adjust contras\nt with volume \nkeys, STOP=End");
	lcd_gotoxy(2,3);
	lcd_bar((contrast-0x20),0x38);
	status_display();
	while(1)
	{
  		wdt_reset();						// reset the watchdog
		time_flag = 0;
		sleepcount = 0;						// power off blocking
		event = get_event();
		switch(event)
		{
			default:
			case EV_IDLE: 
				break;

			case EV_UP:
			case EV_NEXT:
				if (contrast < 0x58)
					contrast++;
				goto SETC;

			case EV_DOWN:
			case EV_PREV:
				if (contrast > 0x20)
					contrast--;
SETC:
				lcd_gotoxy(2,3);
				lcd_bar((contrast-0x20),0x38);
				lcd_contrast(contrast);
				break;

			case EV_STOP:
				eeprom_wb(EEPROM_LCD_CONTRAST,contrast);
				lcd_clrscr();
				set_event(EV_STOP);
				return;
		}
		event = EV_IDLE;
	}
}

#if defined (__AVR_ATmega162__)
 void set_cpu_speed(u08 speed)
 {
	asm volatile("ldi r25, 128");
	asm volatile("cli");
	asm volatile("sts 0x61, r25");
	asm volatile("sts 0x61, r24");
	asm volatile("sei");
 }
#endif


//----------------------------------------------------------------------------
// Main part
//----------------------------------------------------------------------------
int main(void) 
{
 
 //------------------------------
 // Initialize 
 //------------------------------

//////////////////////////////////////////////////////////////////
// B - Port
//////////////////////////////////////////////////////////////////

	cbi(PORTB,LED_PIN);
	sbi(DDRB, LED_PIN);		// pin PB0 is output for LED or backlight
	sbi(PORTB,RESET_PIN);		// RESET- hi
	sbi(DDRB, RESET_PIN);		// pin PB1 is output, RESET- 

	// PB2 is BSYNC pin to VS1001 and RST pin to FTDI chip
	// PB3 is MP3 signal to VS1001
	// PB4 is SCE signal for LCD dosplay
	// PB5 (MOSI) is used for SPI
	// PB6 (MISO) is used for SPI
	// PB7 (SCK) is used for SPI
	
//////////////////////////////////////////////////////////////////
// D - Port
//////////////////////////////////////////////////////////////////

	outp(0xFF,PORTD);	// activate pullups
	outp(0xC0,DDRD);
	// pin PD0 is DREQ input
	// pin PD1, PD4 and PD5 is keyboard input, enable pullup
	// PD2 and PD3 are RXF and TXE input pins from FTDI
	// PD6 and PD7 are WR and RD signals for CF and FTDI chip.

//////////////////////////////////////////////////////////////////
// E - Port
//////////////////////////////////////////////////////////////////

	sbi(PORTE,BUTTON_1);			// pin PE0 is keyboard input, enable pullup

#ifndef PCB_REV_D
	cbi(CHARGE_PORT-1,CHARGE_PIN);		// set pin as input
	sbi(CHARGE_PORT,CHARGE_PIN);		// activate pullup
#endif

#if defined (__AVR_ATmega162__)
	outp(0, EMCUCR);
	set_cpu_speed(CLKDIV);			// CPU clock = XTAL / M162CLKDIV
						// eg about 3-4MHz
 #if (BACKLIGHT_TIME != 0)
	outp(0x00, OCR0);			// PWM duty = 0%
	outp((1<<WGM00) | (1<<COM01) | (1<<CS01) , TCCR0);	// Timer 0 normal PWM mode, CLK/8
 #endif
#endif

	outp((1<<SRE), MCUCR);			// enable external SRAM 

	delayms(50);
	usb_init();
	wdt_enable(7);

	//
	// setup timer1 for a 100mS periodic interrupt
	//	
	outp(0, TCCR1A);
	outp(4, TCCR1B);			// prescaler /256  tPeriod = 32 uS
	outp(TI1_H, TCNT1H);			// load counter value hi
	outp(TI1_L, TCNT1L);			// load counter value lo

	sbi(TIMSK, TOIE1);	 		// enable timer1 interrupt

	// setup some buffers
	SectorBuffer = (u08 *) card_sbuf;	// 512 bytes

// *********************************************************************

	u08 tmp,logo;

// *********************************************************************

	sei();
#ifdef LCD_NEGATIV
	lcd_init(1);
#else
	lcd_init(0);
#endif

	tmp = eeprom_rb(EEPROM_LCD_CONTRAST);
	if (tmp > 0x58 || tmp < 0x20)
	{
		tmp = 0x45;
		eeprom_wb(EEPROM_LCD_CONTRAST,tmp);
	}
	lcd_contrast(tmp);
	lcd_clrscr();
 #if (BACKLIGHT_TIME != 0)
	light_mode = eeprom_rb(EEPROM_BACKLIGHT) % 6;
	backlight_on();
 #endif
	logo = 4+lcd_logo(0);
	if (logo == 4)
		lcd_gotoxy(3,logo);
	else
		_p_puts(model_txt);
	puts(" v.");
	_p_puts(version_txt);

	// init VS1001
	cbi(PORTB,RESET_PIN);				// RESET- lo
	delayms(2);
	sbi(PORTB,RESET_PIN);				// RESET- hi
	delayms(10);
	vs1001_init_io();				// init VS1001
	vs1001_setcomp(F_VS1001);
	vs1001_reset(0);

	if ((volume = eeprom_rb(EEPROM_VOLUME)) > MIN_VOLUME)
		volume = MIN_VOLUME / 2;		// half scale
	balance = eeprom_rb(EEPROM_BALANCE) & 0xfe;
	bLoudness = eeprom_rb(EEPROM_LOUDNESS) % 13;
	bRandom = eeprom_rb(EEPROM_RANDOM);
	Repeat = eeprom_rb(EEPROM_REPEAT) % 4;
	timemode = eeprom_rb(EEPROM_TIME);
	autolock = eeprom_rb(EEPROM_AUTOLOCK);
	curIndex = eeprom_rw(EEPROM_SONGPOS);
	curPlaylist = eeprom_rw(EEPROM_PLAYLIST);

	vs1001_setvolume(volume,balance);

	// Init and wait for CARD ready
 	puts("\nInit Card...\n");

#ifdef MMC_CARD
	MMC_Reset();
#else
	ATA_SetDrive(0);
#endif
	delayms(10);
#ifndef FIXED_RESET
	tmp = 4;
	while(tmp--)					// for propper CF init after card removing.
	{
		wdt_reset();
		logo = CARD_SW_Reset2();		// for now logo variable is second temp variable
		if (logo)				// if card is ready
			break;
		else
			beep(SINE_800,50);
	}

	if (!logo)					// if card is not ready
		PowerOff();				// power off
#else
	wdt_reset();
	if (!CARD_SW_Reset2())
	{
		delayms(800);
		PowerOff();				// power off
	}
#endif

#ifndef MMC_CARD
	WriteBYTE(ATA_SectorCount, SLEEP_TIME);		// CF standby time * 5 milisecond (0=disable)
	WriteBYTE(ATAPI_Cmd, 0xE3);			// set CF standby timer and execute IDLE
	ATA_WaitBusy();
#endif
	logo = 4+lcd_card_logo();
	if (logo == 4)
		lcd_clrline(5);
	else
		lcd_gotoxy(0,2);

  	puts("Ready!\n");

	beep(SINE_1500,100);				// Wakeup beep

	outp(0x40,ACSR);				// Enable Analog Comparator and Bandgap reference

	init_stuff();

	InitRnd();					// init randomizer table
	LoudSet();
	if (curPlaylist >= playlist_qty)
		curPlaylist = curIndex = 0;

	delayms(1000);
	wdt_enable(6);

///////////////////////////////////////////////////////////////////////////////
// start things rolling
///////////////////////////////////////////////////////////////////////////////

	event_e event = EV_IDLE;
	YADL_PLAY_LIST_ENTRY *pListEntry;

	set_event(EV_STOP);

	//
	// main loop
	//
  	while (1) 
  	{
		event = get_event();
		switch (event)
		{
			default:
			case EV_IDLE: 
				break;

			case EV_MENU:
				info_flag = 2;				// disable displaying
				menu_event = browse_menu() + 1;

				if(menu_event)
				{
					set_event(menu_event);
					break;
				}
				else
				{
BROWSE_BREAK:				info_flag = 0;			// enable displaying
					menu_event = EV_IDLE;
					if (isPlaying)
						Playlist_name();	// show Playlist name
					else
						set_event(EV_STOP);
					break;
				}

			case EV_CONTRAST:
				set_contrast();
				break;
#if (BACKLIGHT_TIME != 0)
			case EV_LIGHTMODE:
				lcd_gotoxy(0,5);
 #if defined (__AVR_ATmega162__)
				light_mode =(u08)(light_mode + 1) % 6;
				if(light_mode == 0)
					puts("OFF");
				else if(light_mode == 1)
					puts("OFF/DIMM");
				else if(light_mode == 2)
					puts("OFF/ON");
				else if(light_mode == 3)
					puts("DIMM");
				else if(light_mode == 4)
					puts("DIMM/ON");
				else
					puts("ON");
				puts("       ");
 #else
				light_mode =(u08)(light_mode + 1) % 3;
				if(light_mode == 0)
					puts("OFF");
				else if(light_mode == 1)
					puts("AUTO");
				else
					puts("ON");
				puts("   ");
 #endif
				if(light_mode)
					backlight_on();
				else
					backlight_off();
				break;
#endif
			case EV_TIME:
				timemode = !timemode;				// change mode of time displaying (normal<->remain)
				lcd_gotoxy(0,5);
				if(timemode)
					puts("REMAIN ");
				else
					puts("NORMAL ");

				sleepcount = 0;					// clear shutdown timeout count
				break;

			case EV_PLAYLIST:
				event = EV_IDLE;
				info_flag = 2;					// disable displaying
				u16 Index = browse_songs();			// browse songs within playlist
				if (Index == (u16)-1)
					goto BROWSE_BREAK;

				curIndex = Index;
				menu_event = EV_IDLE;
				info_flag = 0;
				InitRnd();
				isPlaying = 0;

			case EV_PLAY:
				if (menu_event)
				{
					set_event(menu_event);
					time_flag = 0;
					continue;
				}

				if (isPlaying)					// Pause function
				{
					while(get_event() != EV_IDLE)		// Wait for release key
						wdt_reset();

					lobatt_time = 0;
					outp(0xA0,MCUCR);			// enable SLEEP mode
					u08 cc = 6;
					isPlaying = 0;
					do 
					{
						asm volatile ("sleep");		// go to sleep
						asm volatile ("nop");
						wdt_reset();
						if (cc-- == 0)
							cc=8;

						lcd_gotoxy(8,3);		// position for "Paused" string
						if (cc > 4)
						{
#if (BACKLIGHT_TIME == 0)
							cbi(PORTB,LED_PIN);	// LED off
#endif
							asm volatile ("nop");
						}
						else
						{
#if (BACKLIGHT_TIME == 0)
							sbi(PORTB,LED_PIN);	// LED on
#endif
							lcd_invert();		// set white on black mode
						}
						puts("Paused");			// printf string on LCD
						lcd_normal();			// set black on white mode
						time_flag = 1;

						if (lobatt_time > MAX_PAUSE_TIME || !CARD_SW_Reset2())
							PowerOff();		// max 15 minutes of pause

						get_key();

					} while (check_event() != EV_PLAY);	// stay within loop until play pressed again


					isPlaying = 1;
#if (BACKLIGHT_TIME == 0)
					sbi(PORTB,LED_PIN);			// LED On
#endif
					time_flag = 5;
					break;
				}
				else						// Play function
				{
					BatteryMeasure();
					
					if (lobatt)				// low battery
					{
						lcd_gotoxy(0,logo);
						puts("Low Battery !");
						send_sinewave_beeps();
						delayms(1000);
						PowerOff();
					}

					if(curPlaylist < playlist_qty && playlist_qty > 0)
					{
						pListEntry = get_pl_entry(curPlaylist);
						if(curIndex >= pListEntry->entry_qty)
							curIndex = 0;
						play_song();
					}
					break;
				}
/*
			case EV_PLAY_USB:
				lcd_gotoxy(0,logo);
				puts("Playing via\nUSB");
				usb_play = 1;
				isPlaying = 1;
#if (BACKLIGHT_TIME == 0)
				sbi(PORTB, LED_PIN);				// LED ON
#endif
				vs1001_reset(0);
				LoudSet();
				break;
*/
			case EV_STOP:
				if (menu_event)
				{
					time_flag = 40;
					break;
				}
				if (isPlaying)
				{
					sleepcount = 0;			// clear shutdown timeout count
					vs1001_reset(0);
					vs1001_write(0x0A,0);
					InitRnd();			// init randomizer table
					isPlaying = 0;
					usb_play = 0;
				}
//				vs1001_write(0x0B,0xffff);		// turn off analog part of VS1001, generate a click
				info_flag = 0;
				lcd_clrscr();
				if(lcd_card_logo())
				{
					lcd_gotoxy(1,2);
					puts("yampp-7 ");
				}
				else
					lcd_gotoxy(0,4);
				puts("STOP");
				BatteryMeasure();
				
				if (lobatt)
					lobatt = 2;
				status_display();
#if (BACKLIGHT_TIME == 0)
				cbi(PORTB,LED_PIN);				// Led Off
#endif
				if(usb_link)
				{
					lcd_gotoxy(0,logo-1);
					puts("\nPC Link Active\n");
					eeprom_update();
					usb_response(0);
					lobatt_time = 0;
#if defined (__AVR_ATmega162__)
 #ifndef MMC_CARD
					outp(0xC0, MCUCR);			// enable external SRAM 1 waitstate
 #endif
					set_cpu_speed(0);			// CPU clock = XTAL
#endif										// eg about 8-14MHz
				}
				break;

			case EV_BALANCE:
				if (isPlaying)
				{
					info_flag = 1;
					time_flag = 0;
					Playlist_name();
				}
				break;

			case EV_DOWN:						// volume down (or balance left)
				if (isPlaying)
				{
					if(info_flag == 1)
					{
						time_flag = 0;
#ifdef REVERSE_BALANCE
						if (balance < 57)
							balance +=2;
#else
						if (balance > -57)
							balance -=2;
#endif
					}
					else
					{
						if (volume <= MIN_VOLUME-VOL_STEP) 
							volume += VOL_STEP;
						else
							volume = MIN_VOLUME;
					}
					goto SETVOL;
				}
				else if (!menu_event)
					set_event(EV_CONTRAST);
				break;

			case EV_UP: 						// volume up (or balance right)
				if (isPlaying)
				{
					if(info_flag == 1)
					{
						time_flag = 0;
#ifdef REVERSE_BALANCE
						if (balance > -57)
							balance -=2;
#else
						if (balance < 57)
							balance +=2;
#endif
					}
					else
					{
						if (volume >= VOL_STEP) 
							volume -= VOL_STEP;
						else
							volume = 0;
					}
SETVOL:					vs1001_setvolume(volume,balance);
					status_display();
					break;
				}
#if (AUTOLOCK_TIME != 0)
				else  if (!menu_event)
					set_event(EV_AUTOLOCK);
				break;

			case EV_AUTOLOCK:
					autolock = !autolock;
					status_display();
					sleepcount = 0;			// clear shutdown timeout count
					if (!menu_event)
						delayms(500);
#endif
					break;

			case EV_NEXT:
				if (info_flag == 1)
				{
					set_event(EV_UP);		// if balance control active
					break;
				}
				if (menu_event)
				{
					set_event(EV_MENU);		// if menu is active
					break;
				}
				if (isPlaying)
					next_song(1);			// while playing
				else 
					set_event(EV_MBASS);		// while stop
				break;

			case EV_PREV:
				if (info_flag == 1)
				{
					set_event(EV_DOWN);		// if balance control active
					break;
				}
				if (menu_event)
				{
					set_event(EV_MENU);		// if menu active
					break;
				}
				if (isPlaying)
					next_song(2);			// while playing
				else
					set_event(EV_RANDOM);		// while stop
				break;

			case EV_NEXTPL:
				next_playlist();
				goto PLCHG;

			case EV_PREVPL:
				prev_playlist();
				if (!bRandom)
					curIndex = 0;			
PLCHG:				if (isPlaying)
					play_song();
				Playlist_name();
				break;

			case EV_FFWD:
				if (isPlaying && !usb_play)
				{
					MyCluster = next_fat();		// get next cluster position
					init_load_sectors(MyCluster);		// skip to next cluster

					// calculate new size of played data
					fileplayed = ClusterCount * YADL_CLUSTERSIZE / 32;

					// calculation of playing time - small hazard :-)
					addtime = (u16)(fileplayed*32 / (bitrate * 125)) - 
							vs1001_read(VS1001_PLAYTIME); 

					load_sector();
					outptr = SectorBuffer;			// play from buffer 1
					vs1001_setvolume(volume+40,balance);	// volume down during search
					vol_decrease = 1;			// setting of volume downed flag
					time_flag = 5;
				}
				else if (!isPlaying && !menu_event)
					PowerOff();					// while stop

				break;

			case EV_MBASS:
				if (vs1001_fw == 0x4020)
					bLoudness = (bLoudness+1) % 9;
				else if (vs1001_fw == 0x4200)
					bLoudness = (bLoudness+1) % 13;
				else
					bLoudness = !bLoudness;
				LoudSet();
				status_display();
				sleepcount = 0;					// clear shutdown timeout count
				break;

			case EV_RANDOM:
				bRandom = !bRandom;
				status_display();
				sleepcount = 0;					// clear shutdown timeout count
				break;

			case EV_REPEAT:
				Repeat = (Repeat + 1) % 4;
				status_display();
				sleepcount = 0;					// clear shutdown timeout count
				break;
		}
		event = EV_IDLE;
  	} // while (1)    
}
